/****************************************************************************/
/*                               Calendrier                                 */
/*           (C) juin 1999 Nicolas GEORGE <nicolas.george@ens.fr>           */
/*                                                                          */
/*  Ce programme est libre. Vous avez le droit de vous en servir comme bon  */
/*  vous semble, suivant la license publique gnrale GNU (que je n'inclue  */
/*  pas pour ne pas tripler la taille).                                     */
/*  Je ne garantie videmment rien quant  ce que fait ce programme.        */
/****************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include "calendar.inc"

typedef struct {
  int j,m,a,js;
} jour;

typedef struct {
  char *m[12];
  char *j[7];
  char *jc[7];
} noms_langue;

int lmoi[12]={31,29,31,30,31,30,31,31,30,31,30,31};
int dmoi[12]={0,31,59,90,120,151,181,212,243,273,304,334};

#define FRANCAIS 0
#define ENGLISH 1
#define DEUTSCH 2

#ifndef LANGUE
#define LANGUE FRANCAIS
#endif

noms_langue *nl,noms[3]={
  { {"Janvier","Fvrier","Mars","Avril","Mai","Juin","Juillet",
     "Aot","Septembre","Octobre","Novembre","Dcembre"},
    {"Dimanche","Lundi","Mardi","Mercredi","Jeudi","Vendredi","Samedi"},
    {"Dim","Lun","Mar","Mer","Jeu","Ven","Sam"}},

  { {"January","February","Mach","April","May","June",
     "July","August","September","October","November","December"},
    {"Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"},
    {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"}},

  { {"Januar","Februar","Mrz","April","Mag","Juni",
     "Juli","August","September","Oktober","Dezember","November"},
    {"Sonntag","Montag","Dienstag","Mittwoch","Donnerstag","Freitag","Samstag"},
    {"Son","Mon","Die","Mit","Don","Fre","Sam"}}
};


int langue=LANGUE;

void set_font(char *f,int t)
{
  printf("/F0 %d/%s@0 SF\n",t,f);
}

void print_string(int x,int y,char *s)
{
  char c;
  putchar('(');
  for(;*s!=0;s++){
    c=*s;
    if((c=='(')||(c==')')||(c=='\\'))
      putchar('\\');
    putchar(c);
  }
  printf(") %d %d Q\n",x,y);
}

void center_string(int x,int y,char *s)
{
  char c;
  putchar('(');
  for(;*s!=0;s++){
    c=*s;
    if((c=='(')||(c==')')||(c=='\\'))
      putchar('\\');
    putchar(c);
  }
  printf(") %d %d U\n",x,y);
}

void begin_line(int x,int y)
{
  printf("%d %d moveto ",x,y);
}

void draw_line(int x,int y)
{
  printf("%d %d lineto ",x,y);
}

void end_line(void)
{
  printf("stroke\n");
}

void draw_rectangle(int x,int y,int l,int h)
{
  printf("%d %d moveto %d %d lineto %d %d lineto %d %d lineto %d %d lineto stroke\n",
	 x,y,x+l,y,x+l,y+h,x,y+h,x,y);
}

int isbis(int a)
{
  return(((a&3)==0)&&((a%100!=0)||(a%400==0)));
}

long numjour(int j,int m,int a)
{
  m--;
  return(365L*a+(a>>2)-(a/100)+(a/400)+dmoi[m]+j-((m<2)&&isbis(a)));
}

jour *valjour(long n)
{
  static jour r;

  r.js=(n+6)%7;
  r.a=400*(n/146097L);
  n%=146097;
  r.a+=100*((n-1)/36524L);
  if(n>36524L){
    n=(n-36525L)%36524L;
    if(n>58) n++;
  }
  r.a+=4*(n/1461);
  n%=1461;
    r.a+=(n-1)/365L;
  if(n>365L){
    n=(n-366)%365;
    if(n>58) n++;
  }
  r.m=n/31;
  r.j=n-dmoi[r.m]-(r.m>1)+1;
  if(r.j>lmoi[r.m]){
    r.j-=lmoi[r.m];
    r.m++;
  }
  r.m++;
  return(&r);
}

void creer_mois(int p,int m,int a)
{
  char b[40];
  int lm,ns,lc,jb,x,y,x0,i;
  long jbt;
  jour jo;

  printf(ps_page_head,p,p);

  if((m<1)||(m>12)){
    fprintf(stderr,"Mois impossible, page saute\n");
    printf(ps_page_trail);
    return;
  }

  set_font("Times-Roman",32);
  sprintf(b,"%s %d",nl->m[m-1],a);
  center_string(297,60,b);
  lm=lmoi[m-1]-((m==2)&&!isbis(a));
  jbt=numjour(1,m,a);
  jo=*valjour(jbt);
  jb=jo.js;
  ns=(jb+lm+6)/7;
  lc=ns==6?94:108;
  x0=(594-lc*ns)/2;
  set_font("Times-Roman",14);
  for(i=0;i<lm;i++){
    y=(jb+i)%7;
    x=(jb+i)/7;
    x=x0+lc*x;
    y=80+96*y;
    draw_rectangle(x,y,lc,96);
    sprintf(b,"%s %d",nl->j[(jb+i)%7],i+1);
    print_string(x+8,y+14,b);
  }
  printf(ps_page_trail);
}

void creer_annee(int p,int a)
{
  char b[40];
  int i,j,la,x,y,jbm=0;
  long jb;
  jour jo;

  printf(ps_page_head,p,p);
  set_font("Times-Roman",32);
  sprintf(b,"%d",a);
  center_string(297,60,b);
  set_font("Times-Roman",14);
  for(i=0;i<12;i++){
    sprintf(b,"%s",nl->m[i]);
    center_string(109+188*(i%3),94+170*(i/3),b);
  }
  set_font("Times-Roman",8);
  for(i=0;i<12;i++){
    x=34+188*(i%3);
    y=106+170*(i/3);
    for(j=0;j<7;j++,x+=25)
      center_string(x,y,nl->jc[j]);
  }
  jb=numjour(1,1,a);
  la=365+isbis(a);
  set_font("Times-Roman",10);
  for(i=0;i<la;i++){
    jo=*valjour(jb+i);
    if(jo.j==1)
      jbm=jo.js;
    x=21+188*((jo.m-1)%3)+25*jo.js;
    y=120+174*((jo.m-1)/3)+22*((jo.j-1+jbm)/7);
    sprintf(b,"%d",jo.j);
    center_string(x+12,y,b);
  }
  printf(ps_page_trail);
}

int main(int argc,char **argv)
{
  int i,j,p;

  nl=noms+LANGUE;
  for(i=1,j=0;i<argc;i++)
    if(argv[i][0]!='-')
      j++;
  if(j==0){
    fprintf(stderr,"Aucune page\n");
    return(0);
  }
  printf(ps_head,j);
  for(i=1,p=1;i<argc;i++){
    if(argv[i][0]=='-'){
      switch(argv[i][1]){
        case 'f': nl=noms+FRANCAIS; break;
        case 'e': nl=noms+ENGLISH; break;
        case 'd': nl=noms+DEUTSCH; break;
        default:
	  fprintf(stderr,"Option %s non reconnue ignore\n",argv[i]);
      }
    }else{
      for(j=0;argv[i][j]!=0;j++)
	if(argv[i][j]=='/')
	  break;
      if(argv[i][j]=='/'){
	creer_mois(p++,atoi(argv[i]),atoi(argv[i]+j+1));
      }else{
	creer_annee(p++,atoi(argv[i]));
      }
    }
  }
  printf(ps_trail);
  return(0);
}
